#include "compose.h"
#include <string>
#include <stdio.h>
#include "LibHolo.h"

using namespace std;
using namespace LibHolo;

#include "ruby.h"

VALUE ComposeTLSyllable(int inputType, int outputType, const char *syllable, int forcePOJStyle)
{
	int c;
	bool composing = false;
	HoloSyllable syl;
	string output;
	
	if (inputType != 0 && inputType != 1)
		syl.setInputType(TLSyllable);
	else
		syl.setInputType((SyllableType)inputType);
	
	while (1) {
		c = *syllable++;
		
		if (isalpha(c)) {
			if (!composing) composing = true;
			syl.insertCharacterAtCursor(c);
		}
		else {
			if (composing) {
				int finaltone = 0;
				int emitchar = 0;
				
				// some UNIX doesn't have isnumber(c)
				if (c >= '0' && c <= '9') {
					finaltone = c - '0';
				}
				else {
					// since it's non-numeric char, we'll emit it after the composed syllable
					emitchar = c;
				}
				
				syl.normalize(finaltone);
				
				if (outputType == POJSyllable)
					output += syl.convertToPOJSyllable().composedForm();
				else {
                    HoloSyllable tl = syl.convertToTLSyllable();
                    tl.setForcePOJStyle(forcePOJStyle ? true : false);    
					output += tl.composedForm();
				}
				
				if (emitchar) {
					output += string(1, emitchar);
				}
				
				composing = false;
				syl.clear();
			}
			else {
				if (c) output += string(1, c);
			}
		}
		
		if (!c) break;
	}
	
	return rb_str_new2(output.c_str());
}
